pub struct System;Expand description
The default memory allocator provided by the operating system.
This is based on malloc on Unix platforms and HeapAlloc on Windows,
plus related functions. However, it is not valid to mix use of the backing
system allocator with System, as this implementation may include extra
work, such as to serve alignment requests greater than the alignment
provided directly by the backing system allocator.
This type implements the GlobalAlloc trait. Currently the default
global allocator is unspecified. Libraries, however, like cdylibs and
staticlibs are guaranteed to use the System by default and as such
work as if they had this definition:
use std::alloc::System;
#[global_allocator]
static A: System = System;
fn main() {
let a = Box::new(4); // Allocates from the system allocator.
println!("{a}");
}You can also define your own wrapper around System if youβd like, such as
keeping track of the number of all bytes allocated:
use std::alloc::{System, GlobalAlloc, Layout};
use std::sync::atomic::{AtomicUsize, Ordering::Relaxed};
struct Counter;
static ALLOCATED: AtomicUsize = AtomicUsize::new(0);
unsafe impl GlobalAlloc for Counter {
unsafe fn alloc(&self, layout: Layout) -> *mut u8 {
let ret = unsafe { System.alloc(layout) };
if !ret.is_null() {
ALLOCATED.fetch_add(layout.size(), Relaxed);
}
ret
}
unsafe fn dealloc(&self, ptr: *mut u8, layout: Layout) {
unsafe { System.dealloc(ptr, layout); }
ALLOCATED.fetch_sub(layout.size(), Relaxed);
}
}
#[global_allocator]
static A: Counter = Counter;
fn main() {
println!("allocated bytes before main: {}", ALLOCATED.load(Relaxed));
}It can also be used directly to allocate memory independently of whatever
global allocator has been selected for a Rust program. For example if a Rust
program opts in to using jemalloc as the global allocator, System will
still allocate memory using malloc and HeapAlloc.